//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include <stdlib.h>

#include "WinUtils.h"

#define max(a,b)   (((a)>(b)) ? (a) : (b)) 
#define min(a,b)   (((a)<(b)) ? (a) : (b))

#pragma package(smart_init)

namespace Winutils
{

//---------------------------------------------------------------------------
int SlyMsg( HWND hApp, DWORD dwFlags, LPSTR Caption, LPSTR sz,...)
{
  static TCHAR tach[2000];
  va_list va;
  va_start(va, sz);
  wvsprintf(tach, sz, va);
  va_end(va);
  return MessageBox( hApp, tach, Caption, dwFlags );
}

//---------------------------------------------------------------------------
int SlyErrMsg( HWND hApp, HRESULT hr, DWORD dwFlags, LPSTR Caption, LPSTR sz,...)
{
  char *lpMsgBuf;

  FormatMessage(
    FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
    NULL,
    hr,
    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
    (LPTSTR) &lpMsgBuf,
    0,
    NULL
  );

  static TCHAR tach[2000];
  va_list va;
  va_start(va, sz);
  wvsprintf(tach, sz, va);
  va_end(va);

  strcat( tach, "\n" );  
  strcat( tach, lpMsgBuf );
  LocalFree( lpMsgBuf );
  return MessageBox( hApp, tach, Caption, dwFlags );
}

//---------------------------------------------------------------------------
HFONT __fastcall CreateRotatedFont( TFont *F, int Angle)
{
  LOGFONT LF;

  ZeroMemory( &LF, sizeof(LF) );
  LF.lfHeight           = F->Height;
  LF.lfWidth            = 0;
  LF.lfEscapement       = Angle * 10;
  LF.lfOrientation      = Angle * 10; // = 0;

  if( F->Style.Contains(fsBold) )
    LF.lfWeight         = FW_BOLD;
  else
    LF.lfWeight         = FW_NORMAL;

  LF.lfItalic           = F->Style.Contains(fsItalic);
  LF.lfUnderline        = F->Style.Contains(fsUnderline);
  LF.lfStrikeOut        = F->Style.Contains(fsStrikeOut);
  LF.lfCharSet          = DEFAULT_CHARSET;

  strcpy( LF.lfFaceName, F->Name.c_str() );

  LF.lfQuality          = DEFAULT_QUALITY;
  LF.lfOutPrecision     = OUT_DEFAULT_PRECIS;
  LF.lfClipPrecision    = CLIP_DEFAULT_PRECIS;

  switch( F->Pitch ){
  case fpVariable: LF.lfPitchAndFamily = VARIABLE_PITCH; break;
  case fpFixed   : LF.lfPitchAndFamily = FIXED_PITCH;    break;
  default:         LF.lfPitchAndFamily = DEFAULT_PITCH;
  }

  return( CreateFontIndirect( &LF ) );
}

//---------------------------------------------------------------------------
/*
 Gradient fill procedure - displays a gradient beginning with a chosen    }
 color and ending with another chosen color. Based on TGradientFill       }
 component source code written by Curtis White, cwhite@teleport.com.      }
*/

void __fastcall GradientFillRect(TCanvas* Canvas,
                               TRect &ARect,
                               TColor StartColor,
                               TColor EndColor,
                               TFillDirection Direction,
                               int Colors)
{
  char StartRGB[3];  // Start RGB values
  int  RGBDelta[3];  // Difference between start and end RGB values
  TRect ColorBand;   // Color band rectangular coordinates
  int Delta;
  HBRUSH Brush;

  if( IsRectEmpty(&ARect) )  return;
  if( Colors < 2 ){
    Brush = CreateSolidBrush( ColorToRGB(StartColor) );
    FillRect( Canvas->Handle, &ARect, Brush );
    DeleteObject( Brush );
    return;
  }

  StartColor = (TColor)ColorToRGB(StartColor);
  EndColor   = (TColor)ColorToRGB(EndColor);

  switch( Direction ){
  case fdTopToBottom:
  case fdLeftToRight:
    {
      // Set the Red, Green and Blue colors
      StartRGB[0] = GetRValue(StartColor);
      StartRGB[1] = GetGValue(StartColor);
      StartRGB[2] = GetBValue(StartColor);
      // Calculate the difference between begin and end RGB values
      RGBDelta[0] = GetRValue(EndColor) - StartRGB[0];
      RGBDelta[1] = GetGValue(EndColor) - StartRGB[1];
      RGBDelta[2] = GetBValue(EndColor) - StartRGB[2];
    }
    break;
  case fdBottomToTop:
  case fdRightToLeft:
    {
      // Set the Red, Green and Blue colors
      // Reverse of TopToBottom and LeftToRight directions
      StartRGB[0] = GetRValue(EndColor);
      StartRGB[1] = GetGValue(EndColor);
      StartRGB[2] = GetBValue(EndColor);
      // Calculate the difference between begin and end RGB values
      // Reverse of TopToBottom and LeftToRight directions
      RGBDelta[0] = GetRValue(StartColor) - StartRGB[0];
      RGBDelta[1] = GetGValue(StartColor) - StartRGB[1];
      RGBDelta[2] = GetBValue(StartColor) - StartRGB[2];
    }
    break;
  }

  // Calculate the color band's coordinates
  ColorBand = ARect;
  if( Direction == fdTopToBottom || Direction == fdBottomToTop ){
    Colors = max(2, min((int)Colors, HeightOf(ARect)));
    Delta = HeightOf(ARect) / Colors;
  }else{
    Colors = max(2, min((int)Colors, WidthOf(ARect)));
    Delta = WidthOf(ARect) / Colors;
  }

  // Set the pen style and mode
  Canvas->Pen->Style = psSolid;
  Canvas->Pen->Mode = pmCopy;

  // Perform the fill
  if( Delta > 0 ){
    for( int i=0; i <= Colors; i++ ){
      switch( Direction ){
      // Calculate the color band's top and bottom coordinates
      case fdTopToBottom:
      case fdBottomToTop:
        {
          ColorBand.Top = ARect.Top + i * Delta;
          ColorBand.Bottom = ColorBand.Top + Delta;
        }
        break;
      // Calculate the color band's left and right coordinates
      case fdLeftToRight:
      case fdRightToLeft:
        {
          ColorBand.Left = ARect.Left + i * Delta;
          ColorBand.Right = ColorBand.Left + Delta;
        }
        break;
      }

      // Calculate the color band's color
      Brush = CreateSolidBrush(RGB(
        StartRGB[0] + MulDiv(i, RGBDelta[0], Colors - 1),
        StartRGB[1] + MulDiv(i, RGBDelta[1], Colors - 1),
        StartRGB[2] + MulDiv(i, RGBDelta[2], Colors - 1)));
      FillRect( Canvas->Handle, &ColorBand, Brush );
      DeleteObject( Brush );
    }
  }

  if( Direction == fdTopToBottom || Direction == fdBottomToTop ){
    Delta = HeightOf(ARect) % Colors;
  }else{
    Delta = WidthOf(ARect) % Colors;
  }

  if( Delta > 0 ){
    switch( Direction ){
    // Calculate the color band's top and bottom coordinates
    case fdTopToBottom:
    case fdBottomToTop:
      {
        ColorBand.Top = ARect.Bottom - Delta;
        ColorBand.Bottom = ColorBand.Top + Delta;
      }
      break;
    // Calculate the color band's left and right coordinates
    case fdLeftToRight:
    case fdRightToLeft:
      {
        ColorBand.Left = ARect.Right - Delta;
        ColorBand.Right = ColorBand.Left + Delta;
      }
      break;
    }

    switch( Direction ){
    case fdTopToBottom:
    case fdLeftToRight:
      Brush = CreateSolidBrush(EndColor);
      break;
    default:
      // fdBottomToTop, fdRightToLeft
      Brush = CreateSolidBrush(StartColor);
      break;
    }
    FillRect( Canvas->Handle, &ColorBand, Brush );
    DeleteObject( Brush );
  }
}

//---------------------------------------------------------------------------
AnsiString __fastcall GetWindowsVersion( void )
{
  OSVERSIONINFO Ver;
  char Platform[4];

  Ver.dwOSVersionInfoSize = sizeof(Ver);
  GetVersionEx( &Ver );

  switch( Ver.dwPlatformId ){
  case VER_PLATFORM_WIN32s:
    strcpy(Platform,"32s");
  break;
  case VER_PLATFORM_WIN32_WINDOWS:
    {
      Ver.dwBuildNumber &= 0xFFFF;
      if( (Ver.dwMajorVersion >  4) ||
          (Ver.dwMajorVersion == 4 && Ver.dwMinorVersion >= 10) ){
        strcpy(Platform, "98");
      }else{
        strcpy(Platform, "95");
      }
    }
  break;
  case VER_PLATFORM_WIN32_NT:
    strcpy( Platform, "NT");
  break;
  }

  TVarRec tmp[5] = { Platform,
                     (int)Ver.dwMajorVersion,
                     (int)Ver.dwMinorVersion,
                     (int)Ver.dwBuildNumber,
                     Ver.szCSDVersion };

  return( Trim( Format("Windows %s (v%d.%.2d build %.3d %s)", tmp, 5) ) );
}

//---------------------------------------------------------------------------
void __fastcall ShowLastError( char *Caption )
{
  char *lpMsgBuf;

  FormatMessage(
    FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
    NULL,
    GetLastError(),
    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
    (LPTSTR) &lpMsgBuf,
    0,
    NULL
  );

  Application->MessageBox( lpMsgBuf, Caption, MB_OK | MB_ICONINFORMATION );

  LocalFree( lpMsgBuf );
}

//---------------------------------------------------------------------------

}  // namespace

//---------------------------------------------------------------------------

